@@ -79,14 +79,20 @@ class MesonMain(MesonInterpreterObject):
                              })
 
     def _find_source_script(
-            self, prog: T.Union[str, mesonlib.File, build.Executable, ExternalProgram],
+            self, name: str, prog: T.Union[str, mesonlib.File, build.Executable, ExternalProgram],
             args: T.List[str]) -> 'ExecutableSerialisation':
         largs: T.List[T.Union[str, build.Executable, ExternalProgram]] = []
+
         if isinstance(prog, (build.Executable, ExternalProgram)):
+            FeatureNew.single_use(f'Passing executable/found program object to script parameter of {name}',
+                                  '0.55.0', self.subproject, location=self.current_node)
             largs.append(prog)
             largs.extend(args)
             return self.interpreter.backend.get_executable_serialisation(largs)
-        found = self.interpreter.func_find_program({}, prog, {})
+        elif isinstance(prog, mesonlib.File):
+            FeatureNew.single_use(f'Passing file object to script parameter of {name}',
+                                  '0.57.0', self.subproject, location=self.current_node)
+        found = self.interpreter.find_program_impl([prog])
         largs.append(found)
         largs.extend(args)
         es = self.interpreter.backend.get_executable_serialisation(largs)
@@ -95,10 +101,10 @@ class MesonMain(MesonInterpreterObject):
 
     def _process_script_args(
             self, name: str, args: T.Sequence[T.Union[
-                str, mesonlib.File, build.Target,
+                str, mesonlib.File, build.BuildTarget, build.CustomTarget,
                 build.CustomTargetIndex,
-                ExternalProgram, build.Executable,
-            ]], allow_built: bool = False) -> T.List[str]:
+                ExternalProgram,
+            ]]) -> T.List[str]:
         script_args = []  # T.List[str]
         new = False
         for a in args:
@@ -107,9 +113,7 @@ class MesonMain(MesonInterpreterObject):
             elif isinstance(a, mesonlib.File):
                 new = True
                 script_args.append(a.rel_to_builddir(self.interpreter.environment.source_dir))
-            elif isinstance(a, (build.Target, build.CustomTargetIndex)):
-                if not allow_built:
-                    raise InterpreterException(f'Arguments to {name} cannot be built')
+            elif isinstance(a, (build.BuildTarget, build.CustomTarget, build.CustomTargetIndex)):
                 new = True
                 script_args.extend([os.path.join(a.get_subdir(), o) for o in a.get_outputs()])
 
@@ -127,7 +131,7 @@ class MesonMain(MesonInterpreterObject):
 
         if new:
             FeatureNew.single_use(
-                f'Calling "{name}" with File, CustomTaget, Index of CustomTarget, '
+                f'Calling "{name}" with File, CustomTarget, Index of CustomTarget, '
                 'Executable, or ExternalProgram',
                 '0.55.0', self.interpreter.subproject)
         return script_args
@@ -135,7 +139,7 @@ class MesonMain(MesonInterpreterObject):
     @typed_pos_args(
         'meson.add_install_script',
         (str, mesonlib.File, build.Executable, ExternalProgram),
-        varargs=(str, mesonlib.File, build.Target, build.CustomTargetIndex, ExternalProgram)
+        varargs=(str, mesonlib.File, build.BuildTarget, build.CustomTarget, build.CustomTargetIndex, ExternalProgram)
     )
     @typed_kwargs(
         'meson.add_install_script',
@@ -145,14 +149,10 @@ class MesonMain(MesonInterpreterObject):
     def add_install_script_method(
             self,
             args: T.Tuple[T.Union[str, mesonlib.File, build.Executable, ExternalProgram],
-                          T.List[T.Union[str, mesonlib.File, build.Target, build.CustomTargetIndex, ExternalProgram]]],
+                          T.List[T.Union[str, mesonlib.File, build.BuildTarget, build.CustomTarget, build.CustomTargetIndex, ExternalProgram]]],
             kwargs: 'AddInstallScriptKW') -> None:
-        if isinstance(args[0], mesonlib.File):
-            FeatureNew.single_use('Passing file object to script parameter of add_install_script',
-                                  '0.57.0', self.interpreter.subproject)
-
-        script_args = self._process_script_args('add_install_script', args[1], allow_built=True)
-        script = self._find_source_script(args[0], script_args)
+        script_args = self._process_script_args('add_install_script', args[1])
+        script = self._find_source_script('add_install_script', args[0], script_args)
         script.skip_if_destdir = kwargs['skip_if_destdir']
         script.tag = kwargs['install_tag']
         self.build.install_scripts.append(script)
@@ -160,43 +160,37 @@ class MesonMain(MesonInterpreterObject):
     @typed_pos_args(
         'meson.add_postconf_script',
         (str, mesonlib.File, ExternalProgram),
-        varargs=(str, mesonlib.File, build.CustomTarget, build.CustomTargetIndex)
+        varargs=(str, mesonlib.File, ExternalProgram)
     )
     @noKwargs
     def add_postconf_script_method(
             self,
             args: T.Tuple[T.Union[str, mesonlib.File, ExternalProgram],
-                          T.List[T.Union[str, mesonlib.File, build.CustomTarget, build.CustomTargetIndex]]],
+                          T.List[T.Union[str, mesonlib.File, ExternalProgram]]],
             kwargs: 'TYPE_kwargs') -> None:
-        if isinstance(args[0], mesonlib.File):
-            FeatureNew.single_use('Passing file object to script parameter of add_postconf_script',
-                                  '0.57.0', self.interpreter.subproject)
-        script_args = self._process_script_args('add_postconf_script', args[1], allow_built=True)
-        script = self._find_source_script(args[0], script_args)
+        script_args = self._process_script_args('add_postconf_script', args[1])
+        script = self._find_source_script('add_postconf_script', args[0], script_args)
         self.build.postconf_scripts.append(script)
 
     @typed_pos_args(
         'meson.add_dist_script',
-        (str, mesonlib.File, build.Executable, ExternalProgram),
-        varargs=(str, mesonlib.File, build.CustomTarget, build.CustomTargetIndex)
+        (str, mesonlib.File, ExternalProgram),
+        varargs=(str, mesonlib.File, ExternalProgram)
     )
     @noKwargs
     def add_dist_script_method(
             self,
-            args: T.Tuple[T.Union[str, mesonlib.File, build.Executable, ExternalProgram],
-                          T.List[T.Union[str, mesonlib.File, build.CustomTarget, build.CustomTargetIndex]]],
+            args: T.Tuple[T.Union[str, mesonlib.File, ExternalProgram],
+                          T.List[T.Union[str, mesonlib.File, ExternalProgram]]],
             kwargs: 'TYPE_kwargs') -> None:
         if args[1]:
             FeatureNew.single_use('Calling "add_dist_script" with multiple arguments',
                                   '0.49.0', self.interpreter.subproject)
-        if isinstance(args[0], mesonlib.File):
-            FeatureNew.single_use('Passing file object to script parameter of add_dist_script',
-                                  '0.57.0', self.interpreter.subproject)
         if self.interpreter.subproject != '':
             FeatureNew.single_use('Calling "add_dist_script" in a subproject',
                                   '0.58.0', self.interpreter.subproject)
-        script_args = self._process_script_args('add_dist_script', args[1], allow_built=True)
-        script = self._find_source_script(args[0], script_args)
+        script_args = self._process_script_args('add_dist_script', args[1])
+        script = self._find_source_script('add_dist_script', args[0], script_args)
         self.build.dist_scripts.append(script)
 
     @noPosargs
