@@ -28,6 +28,7 @@ from tempfile import TemporaryDirectory
 import typing as T
 import uuid
 import textwrap
+import copy
 
 from mesonbuild import mlog
 
@@ -61,7 +62,6 @@ __all__ = [
     'OptionKey',
     'dump_conf_header',
     'OptionOverrideProxy',
-    'OptionProxy',
     'OptionType',
     'OrderedSet',
     'PerMachine',
@@ -83,11 +83,11 @@ __all__ = [
     'detect_vcs',
     'do_conf_file',
     'do_conf_str',
-    'do_define',
     'do_replacement',
     'exe_exists',
     'expand_arguments',
     'extract_as_list',
+    'generate_list',
     'get_compiler_for_source',
     'get_filenames_templates_dict',
     'get_library_dirs',
@@ -155,7 +155,7 @@ if os.path.basename(sys.executable) == 'meson.exe':
     python_command = [sys.executable, 'runpython']
 else:
     python_command = [sys.executable]
-_meson_command = None
+_meson_command: T.Optional['ImmutableListProtocol[str]'] = None
 
 class MesonException(Exception):
     '''Exceptions thrown by Meson'''
@@ -230,7 +230,7 @@ def set_meson_command(mainfile: str) -> None:
         mlog.log(f'meson_command is {_meson_command!r}')
 
 
-def get_meson_command() -> T.Optional[T.List[str]]:
+def get_meson_command() -> T.Optional['ImmutableListProtocol[str]']:
     return _meson_command
 
 
@@ -396,7 +396,7 @@ class File(HoldableObject):
     @lru_cache(maxsize=None)
     def from_source_file(source_root: str, subdir: str, fname: str) -> 'File':
         if not os.path.isfile(os.path.join(source_root, subdir, fname)):
-            raise MesonException('File %s does not exist.' % fname)
+            raise MesonException(f'File {fname} does not exist.')
         return File(False, subdir, fname)
 
     @staticmethod
@@ -425,7 +425,7 @@ class File(HoldableObject):
     def suffix(self) -> str:
         return os.path.splitext(self.fname)[1][1:].lower()
 
-    def endswith(self, ending: str) -> bool:
+    def endswith(self, ending: T.Union[str, T.Tuple[str, ...]]) -> bool:
         return self.fname.endswith(ending)
 
     def split(self, s: str, maxsplit: int = -1) -> T.List[str]:
@@ -1132,7 +1132,7 @@ def do_replacement(regex: T.Pattern[str], line: str, variable_format: str,
 def do_define(regex: T.Pattern[str], line: str, confdata: 'ConfigurationData', variable_format: str) -> str:
     def get_cmake_define(line: str, confdata: 'ConfigurationData') -> str:
         arr = line.split()
-        define_value=[]
+        define_value = []
         for token in arr[2:]:
             try:
                 (v, desc) = confdata.get(token)
@@ -1179,13 +1179,13 @@ def get_variable_regex(variable_format: str = 'meson') -> T.Pattern[str]:
         raise MesonException(f'Format "{variable_format}" not handled')
     return regex
 
-def do_conf_str (src: str, data: list, confdata: 'ConfigurationData', variable_format: str,
-                 encoding: str = 'utf-8') -> T.Tuple[T.List[str],T.Set[str], bool]:
-    def line_is_valid(line : str, variable_format: str) -> bool:
+def do_conf_str(src: str, data: list, confdata: 'ConfigurationData', variable_format: str,
+                encoding: str = 'utf-8') -> T.Tuple[T.List[str], T.Set[str], bool]:
+    def line_is_valid(line: str, variable_format: str) -> bool:
         if variable_format == 'meson':
             if '#cmakedefine' in line:
                 return False
-        else: #cmake format
+        else: # cmake format
             if '#mesondefine' in line:
                 return False
         return True
@@ -1206,7 +1206,7 @@ def do_conf_str (src: str, data: list, confdata: 'ConfigurationData', variable_f
             confdata_useless = False
             line = do_define(regex, line, confdata, variable_format)
         else:
-            if not line_is_valid(line,variable_format):
+            if not line_is_valid(line, variable_format):
                 raise MesonException(f'Format error in {src}: saw "{line.strip()}" when format set to "{variable_format}"')
             line, missing = do_replacement(regex, line, variable_format, confdata)
             missing_variables.update(missing)
@@ -1319,11 +1319,11 @@ def extract_as_list(dict_object: T.Dict[_T, _U], key: _T, pop: bool = False) ->
     '''
     Extracts all values from given dict_object and listifies them.
     '''
-    fetch = dict_object.get
+    fetch: T.Callable[[_T], _U] = dict_object.get
     if pop:
         fetch = dict_object.pop
     # If there's only one key, we don't return a list with one element
-    return listify(fetch(key, []), flatten=True)
+    return listify(fetch(key) or [], flatten=True)
 
 
 def typeslistify(item: 'T.Union[_T, T.Sequence[_T]]',
@@ -1671,7 +1671,7 @@ class TemporaryDirectoryWinProof(TemporaryDirectory):
 
 
 def detect_subprojects(spdir_name: str, current_dir: str = '',
-                       result: T.Optional[T.Dict[str, T.List[str]]] = None) -> T.Optional[T.Dict[str, T.List[str]]]:
+                       result: T.Optional[T.Dict[str, T.List[str]]] = None) -> T.Dict[str, T.List[str]]:
     if result is None:
         result = {}
     spdir = os.path.join(current_dir, spdir_name)
@@ -1731,9 +1731,7 @@ class OrderedSet(T.MutableSet[_T]):
         return 'OrderedSet()'
 
     def __reversed__(self) -> T.Iterator[_T]:
-        # Mypy is complaining that sets can't be reversed, which is true for
-        # unordered sets, but this is an ordered, set so reverse() makes sense.
-        return reversed(self.__container.keys())  # type: ignore
+        return reversed(self.__container.keys())
 
     def add(self, value: _T) -> None:
         self.__container[value] = None
@@ -1758,6 +1756,10 @@ class OrderedSet(T.MutableSet[_T]):
     def difference(self, set_: T.Union[T.Set[_T], 'OrderedSet[_T]']) -> 'OrderedSet[_T]':
         return type(self)(e for e in self if e not in set_)
 
+    def difference_update(self, iterable: T.Iterable[_T]) -> None:
+        for item in iterable:
+            self.discard(item)
+
 def relpath(path: str, start: str) -> str:
     # On Windows a relative path can't be evaluated for paths on two different
     # drives (i.e. c:\foo and f:\bar).  The only thing left to do is to use the
@@ -1919,14 +1921,12 @@ def run_once(func: T.Callable[..., _T]) -> T.Callable[..., _T]:
     return wrapper
 
 
-class OptionProxy(T.Generic[_T]):
-    def __init__(self, value: _T, choices: T.Optional[T.List[str]] = None):
-        self.value = value
-        self.choices = choices
+def generate_list(func: T.Callable[..., T.Generator[_T, None, None]]) -> T.Callable[..., T.List[_T]]:
+    @wraps(func)
+    def wrapper(*args: T.Any, **kwargs: T.Any) -> T.List[_T]:
+        return list(func(*args, **kwargs))
 
-    def set_value(self, v: _T) -> None:
-        # XXX: should this be an error
-        self.value = v
+    return wrapper
 
 
 class OptionOverrideProxy(collections.abc.MutableMapping):
@@ -1944,15 +1944,16 @@ class OptionOverrideProxy(collections.abc.MutableMapping):
         for o in options:
             self.options.update(o)
 
-    def __getitem__(self, key: 'OptionKey') -> T.Union['UserOption', OptionProxy]:
+    def __getitem__(self, key: 'OptionKey') -> T.Union['UserOption']:
         if key in self.options:
             opt = self.options[key]
             if key in self.overrides:
-                return OptionProxy(opt.validate_value(self.overrides[key]), getattr(opt, 'choices', None))
+                opt = copy.copy(opt)
+                opt.set_value(self.overrides[key])
             return opt
         raise KeyError('Option not found', key)
 
-    def __setitem__(self, key: 'OptionKey', value: T.Union['UserOption', OptionProxy]) -> None:
+    def __setitem__(self, key: 'OptionKey', value: T.Union['UserOption']) -> None:
         self.overrides[key] = value.value
 
     def __delitem__(self, key: 'OptionKey') -> None:
@@ -2126,7 +2127,7 @@ class OptionKey:
         return out
 
     def __repr__(self) -> str:
-        return f'OptionKey({repr(self.name)}, {repr(self.subproject)}, {repr(self.machine)}, {repr(self.lang)})'
+        return f'OptionKey({self.name!r}, {self.subproject!r}, {self.machine!r}, {self.lang!r}, {self.module!r}, {self.type!r})'
 
     @classmethod
     def from_string(cls, raw: str) -> 'OptionKey':
